"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const webpack_node_externals_1 = __importDefault(require("webpack-node-externals"));
const webpack_1 = __importDefault(require("webpack"));
const friendly_errors_webpack_plugin_1 = __importDefault(require("friendly-errors-webpack-plugin"));
const babel_1 = require("./babel");
const utils_1 = require("./utils");
const resolveTsconfigPathsToAlias = ({ compilerOptions } = {}, webpackConfigBasePath = process.cwd()) => {
    if (!compilerOptions || !compilerOptions.paths)
        return {};
    const { paths } = compilerOptions;
    const aliases = {};
    Object.keys(paths).forEach(item => {
        const key = item.replace('/*', '');
        const value = path_1.default.resolve(webpackConfigBasePath, paths[item][0].replace('/*', '').replace('*', ''));
        aliases[key] = value;
    });
    return aliases;
};
const addSourceMapSupportPlugin = new webpack_1.default.BannerPlugin({
    banner: 'require("source-map-support").install();',
    raw: true,
    entryOnly: false
});
exports.getWebpackWebConfig = (options) => {
    const baseConfig = getBaseConfig({ useReact: true, ...options });
    const isDev = process.env.NODE_ENV === 'development';
    return {
        ...baseConfig,
        entry: isDev ? ['react-hot-loader/patch', baseConfig.entry] : baseConfig.entry,
        target: 'web',
        // @ts-ignore
        ...(isDev &&
            {
                devServer: {
                    historyApiFallback: true,
                    port: process.env.PORT || 3000,
                    quiet: true,
                    hot: true
                }
            }),
        resolve: {
            ...baseConfig.resolve,
            alias: {
                ...baseConfig.resolve.alias,
                ...(isDev && { 'react-dom': '@hot-loader/react-dom' })
            }
        },
        output: {
            path: baseConfig.output.path,
            filename: baseConfig.mode === 'development' ? 'app.bundle.js' : 'app.bundle.[hash].js',
            publicPath: '/',
            sourceMapFilename: baseConfig.output.sourceMapFilename
        }
    };
};
exports.getWebpackScriptConfig = (options) => {
    const baseConfig = getBaseConfig(options);
    return {
        ...baseConfig,
        mode: 'development',
        target: 'node',
        externals: [webpack_node_externals_1.default()],
        output: {
            libraryTarget: 'commonjs2',
            path: baseConfig.output.path,
            filename: 'script.js',
            sourceMapFilename: baseConfig.output.sourceMapFilename
        },
        plugins: [addSourceMapSupportPlugin, ...baseConfig.plugins]
    };
};
exports.getWebpackLambdaConfig = (options) => {
    const baseConfig = getBaseConfig(options);
    return {
        entry: [options.serverlessWebpack.lib.entries],
        ...baseConfig,
        target: 'node',
        externals: [webpack_node_externals_1.default()],
        optimization: {
            minimize: false,
            noEmitOnErrors: true
        },
        output: {
            libraryTarget: 'commonjs2',
            path: baseConfig.output.path,
            filename: '[name].js',
            sourceMapFilename: baseConfig.output.sourceMapFilename
        },
        plugins: [addSourceMapSupportPlugin, ...baseConfig.plugins]
    };
};
const getBaseConfig = ({ mode, tsconfig, env, babelOptions, outputFolder, useReact, entry }) => {
    return {
        entry,
        node: false,
        devtool: 'source-map',
        mode,
        performance: { hints: false },
        optimization: { noEmitOnErrors: true },
        stats: 'errors-only',
        module: {
            rules: [
                {
                    test: /\.(js|ts|jsx|tsx)$/,
                    exclude: /node_modules/,
                    use: {
                        loader: 'babel-loader',
                        options: babelOptions || babel_1.getBabelConfig({ useReact })
                    }
                },
                {
                    test: /\.graphql?$/,
                    exclude: /node_modules/,
                    loader: 'webpack-custom-graphql-loader',
                    options: { output: 'document' }
                },
                {
                    test: /\.gql?$/,
                    exclude: /node_modules/,
                    loader: 'webpack-custom-graphql-loader',
                    options: { output: 'string' }
                },
                {
                    test: /\.mjs$/,
                    include: /node_modules/,
                    type: 'javascript/auto'
                }
            ]
        },
        resolve: {
            extensions: ['.ts', '.js', '.graphql', '.json', '.gql'],
            alias: resolveTsconfigPathsToAlias(tsconfig)
        },
        output: {
            path: path_1.default.resolve(process.cwd(), outputFolder || '__build'),
            sourceMapFilename: '[file].map'
        },
        plugins: [new webpack_1.default.EnvironmentPlugin(env || {}), new friendly_errors_webpack_plugin_1.default()]
    };
};
exports.runWithWebpack = (filePath, webpackConfig, watch, callback) => {
    const config = { ...webpackConfig };
    config.watchOptions = {};
    config.entry = filePath;
    const compiler = webpack_1.default(config);
    const handleCompilerOutput = err => {
        if (err) {
            if (callback)
                return callback(err);
            return err;
        }
        const outputPath = path_1.default.join(compiler.options.output.path, compiler.options.output.filename);
        // const packedOutput = memFs.readFileSync(outputPath);
        // eslint-disable-next-line no-eval
        // const compiledObject = eval(packedOutput.toString());
        utils_1.exec({ cmd: 'node', args: [outputPath] });
        if (callback)
            callback(null, {});
    };
    if (watch) {
        compiler.watch({}, handleCompilerOutput);
    }
    else {
        compiler.run(handleCompilerOutput);
    }
};
//# sourceMappingURL=webpack.js.map